# PACO-DCT inpainting

###  Version: 1.0

### Authors

* Ignacio Ramírez Paulino <nacho@fing.edu.uy>
* Ignacio Hounie <ihounie@fing.edu.uy>

### Copyright information

Copyright (c) 2019 Ignacio Ramírez Paulino & Ignacio Hounie

This program is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.


## Compilation instructions

### Dependencies

Development (dev) packages of the following libraries:

* GSL 2.4 or higher
* libpng 1.6 or higher
* zlib1g 

### Procedure

* `make code` to compile the source code; by default compiles in debug mode; use RELEASE=1 make
* `make demo` to run a simple demo
* `make pap` to compile the paper 
* `make api` to generate an API documentation (requires Doxygen)


## Usage 

The program consists of a single executable, `paco_dct_inpainting_tool` which takes an input image, binary mask where pixels with value `1` indicate a _missing_ sample, and `0` indicate a _known_ sample, and an output image filename. 
These are the three minimum required arguments.

### Demo

The root of the project contains a BASH script called `demo.sh`. Running it will:

* download the Kodak image dataset
* compile the project
* run a sample experiment with a given mask on a given image

The demo is well documented and should serve by itself as a starting point to run other experiments.

### Basic

A basic execution will use a default configuration for estimating the pixels marked as missing in `output.png` from the known samples of `input.png`,

    paco_dct_inpainting_tool input.png mask.png output.png

As a side effect, the algorithm will write a file `weights.txt` with the estimated L1 weights for the DCT coefficients. These can be then used as input (see -c switch below) for speeding up subsequent runs.

### Measuring performance with respect to a reference

A fourth optional input image can be specified as a _reference_, in which case metrics such as RMSE and SSIM will be computed at all iterations which are a power of 2. This is the form used for computing the performance measures in the paper.

    paco_dct_inpainting_tool input.png mask.png output.png reference.png

### Program options

Running the algorithm with no arguments will yield a basic usage message. Calling  the program with the `--usage` option shows  a brief description of the available options. Below we describe each one of them with more detail:

#### Main problem parameters

Together with the input image and mask, these define the problem to be solved and are the main parameters to tune if running a demo for example.

* **`-w, --width=patch-width      `** Patch width in both directions. (only square patches are supported for now)
* **`-s, --stride=patch-stride    `** Patch grid stride in both directions. (only uniform grids are supported for now)
* **-`c, --dct-weights=dct-weights`** Path to file containing precomputed DCT weights. By default, the weights are estimated  using all the image patches of size wxw which are complete (not only those in the grid, but all)

#### Optimization parameters

These can be fine-tuned for controlling quality and speed.

* **`-e, --epsilon=small-value             `** Minimum change between iterations.
* **`-m, --max-iter=max-iter               `** Maximum allowed ADMM iterations.
* **`-l, --lambda=real-value               `** ADMM penalty parameter.
* **`-k, --kappa=real-value                `** ADMM penalty parameter multiplier (< 1).
* **`-i, --initialisation=initialisation   `** Optional image to be used as initialization to the algorithm
 
#### Miscelaneous

* **`-q, --quiet`**    Decrease console output
* **`-v, --verbose`**  Increase console output
*   **`--help`**       Give this help list
*   **`--usage`**      Give a short usage message

## Organization

The following folders are defined in the project:

* **`code      `** Implementation of the PACO-DCT algorithm and tools in the C language
* **`scripts   `** BASH scripts for reproducing the results of the paper
* **`data      `** input data: images, masks, etc. Data is not included in the project; scripts are available here to download the files from the project web page.
* **`results   `** the output of the algorithm is stored here by the scripts
* **`doc       `** documentation generated by Doxygen is stored under this directory

### Code organization

The main code is structured into the following C modules (each one has a header `.h` and an implementation `.c`). We further organize them in layers of abstraction/importance as per the paper.

#### User interface

* **paco_dct_inpainting_tool** command line user interface to PACO-DCT

#### Algorithms

* **paco** general solution to Algorithm 1 (PACO) in the paper and that of the specific PACO-DCT inpainting Algorithm 4 when the corresponding functions (see below) are plugged in.
* **paco_dct_inpainting_problem** problem-specific functions  and corresponding proximal operators. These implement the functions defined in Equations (11) and (12) (which in turn is implemented in Algorithm 3).
* **paco_linop** linear operators; in particular, the orthogonal DCT type II transform (FFTW backend)
* **paco_grid_mapping** equal-sized patches defined on a regular grid 

#### Basic types
* **paco_types** scalar types
* **paco_image** multichannel image representation with `gsl_matrix` backend
* **paco_iterate** holds the ADMM iterate variables
 
#### Services and utilities

* **paco_io** image input and output
* **paco_log** debugging  and info
* **paco_metrics** standard performance metrics (declaration)
* *Mmakefile** GNU Makefile for compiling the project
