/*
  * Copyright (c) 2019 Ignacio Francisco Ramírez Paulino and Ignacio Hounie
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Affero General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or (at
  * your option) any later version.
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
  * General Public License for more details.
  * You should have received a copy of the GNU Affero General Public License
  *  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * \file paco_iterate.c
 * \brief iterate structure: implementation
 */
#include "paco_iterate.h"
#include "paco_mapping.h"

paco_iterate_st* paco_iterate_alloc() {
    paco_iterate_st* out = (paco_iterate_st*) calloc(1,sizeof(paco_iterate_st));
    const index_t n = get_num_mapped_patches();
    const index_t m = get_num_patch_samples();
    const index_t M = get_input_height();
    const index_t N = get_input_width();
    out->Y = gsl_matrix_float_alloc(M,N);
    out->B = gsl_matrix_float_alloc(n,m);
    out->A = gsl_matrix_float_alloc(n,m);
    out->U = gsl_matrix_float_alloc(n,m);
    return out;
}

void paco_iterate_free(paco_iterate_st* iter) {
    gsl_matrix_float_free(iter->A);
    gsl_matrix_float_free(iter->B);
    gsl_matrix_float_free(iter->U);
    gsl_matrix_float_free(iter->Y);
    free(iter);
}
