#!/bin/bash
#
#
# Copyright (c) 2019 Ignacio Francisco Ramírez Paulino and Ignacio Hounie
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
# General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# PACO-DCT demo
#
# the arguments for this demo are as follows:
# -w 16        use square patches of size 16x16
# -s 8         use a stride of 8 (25% area overlap)
# -m 256       run up to 256 iterations
# -l 1000      start with a ADMM penalty of 1000
# -k 0.95      the ADMM penalty is multiplied by 0.95 after each iteration
# -$*          the user can override any setting from the command line by passing
#              his/her own arguments to demo.sh
#
ARGS="-w 16 -s 2 -m 256 -e 1e-5 -l 10000 -k 0.99 $*"
#
# set up problem
#
type="maertz"
#
# output directory
#
out="./results/images/kodak/${type}"
#
# metrics per iteration are saved here
#
metrics="./results/metrics/kodak/${type}"
mkdir -p ${out}
mkdir -p ${metrics}
#
# mask, input and output filenames
#
mask="data/image/maertz/parrot_bwmask.png"
input="data/image/maertz/parrot.png"
output="${out}/maertz_parrot.png"
coefs="data/weights/dct_weights_w016_c3.txt"
#
# compile PACO
#
RELEASE=1 make -C code
#
# run PACO
#
#./code/paco_dct_inpaint_tool --dct-weights=${coefs} ${ARGS} ${input} ${mask} ${output} ${input}
./code/paco_dct_inpaint_tool ${ARGS} ${input} ${mask} ${output} ${input}
#
# display the result (requires imagemagick)
#
display ${output}
    
